use actix_files as fs;
use actix_web::{
    get,
    http::header::ContentType,
    web::{Data, Query},
    App, HttpResponse, HttpServer, Responder,
};
use base64::{engine::general_purpose, Engine};
use mime::TEXT_HTML;
use reqwest::{header::HeaderMap, Client};
use serde::{Deserialize, Serialize};
use std::env;
use tera::{Context, Tera};

struct Template {
    paste: Tera,
}

#[derive(Deserialize)]
struct Paste {
    paste_id: Option<String>,
    parse_md: Option<String>,
}

#[derive(Serialize, Deserialize)]
struct GitLabSuccessJSON {
    file_name: String,
    file_path: String,
    size: u64,
    encoding: String,
    content_sha256: String,
    r#ref: String,
    blob_id: String,
    commit_id: String,
    last_commit_id: String,
    execute_filemode: bool,
    content: String,
}

#[derive(Serialize, Deserialize)]
struct GitLabFailedJSON {
    message: String,
}

#[get("/")]
async fn paste_render(data: Data<Template>, paste: Query<Paste>) -> impl Responder {
    let my_data = data.get_ref();
    let mut context = Context::new();
    let gitlab_token = env::var("GITLAB_TOKEN").unwrap_or("".to_string());
    let base_url = env::var("BASE_URL").unwrap_or("https://gitlab.com".to_string());
    let client = Client::new();
    let mut headers = HeaderMap::new();
    headers.insert("PRIVATE-TOKEN", gitlab_token.parse().unwrap());
    let paste_content: String;
    if paste.paste_id.is_some() {
        let response = client.get(format!("{}/repository/files/paste%2F{}.md?ref=master", base_url, paste.paste_id.as_ref().unwrap())).headers(headers).send().await.unwrap();
        let response_code = response.status().as_str().to_string();
        let response_json = response.text().await.unwrap();
        if response_code != "200" {
            let resp_struct: GitLabFailedJSON = serde_json::from_str(&response_json).unwrap();
            if resp_struct.message == "404 Project Not Found" {
                context.insert("md", "true");
                paste_content = "Project ID Mismatch/GitLab Token Missing".to_string();
            }
            else {
                context.insert("md", "true");
                paste_content = "Paste ID is Invalid".to_string();
            }
        } else {
            let resp_struct: GitLabSuccessJSON = serde_json::from_str(&response_json).unwrap();
            paste_content = String::from_utf8(
                general_purpose::STANDARD
                    .decode(resp_struct.content)
                    .unwrap(),
            )
            .unwrap();
        }
    } else {
        context.insert("md", "true");
        paste_content = "This is a no-nonsense Pastebin Server, written in Rust. It serves the content from files on a GitLab Private Repository. Please look into the repository for hosting your own copy of this.".to_string();
    }
    context.insert("paste_content", &paste_content);
    if paste.parse_md.is_some() {
        context.insert("md", "true");
    }
    let rendered = my_data.paste.render("paste.html", &context);
    HttpResponse::Ok()
        .insert_header(ContentType(TEXT_HTML))
        .body(rendered.unwrap())
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    let tera = match Tera::new("templates/*.html") {
        Ok(t) => t,
        Err(e) => {
            println!("Parsing error(s): {}", e);
            ::std::process::exit(1);
        }
    };
    let data = Data::new(Template { paste: tera });
    println!("Paste Server is launched on 0.0.0.0:8080");
    HttpServer::new(move || {
        App::new()
            .app_data(Data::clone(&data))
            .service(paste_render)
            .service(fs::Files::new("/static", "static"))
    })
    .bind(env::var("BIND_ADDRESS").unwrap_or("0.0.0.0:8080".to_string()))?
    .run()
    .await
}
